/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Interface to Dictx
*  SCOPE :              Interface to Dictx
*  FILE :               dictx_tmpl.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __dictx_tmpl_HXX
#define __dictx_tmpl_HXX

/* IMPORTS *******************************************************************/

#ifndef __dictx_HXX
#include "dictx.hxx"
#endif

/*****************************************************************************/


template <class ELEM, class KEYT, long KEY> class DictX : public DictXGen { 
  public : 

  DictX() {} ; 

  DictX(const DictX<ELEM, KEYT, KEY>& s) { *this = s ; } 

  static void* extract(const void* atom) {
    return (void*)&
      ((*(Ref<ELEM>*)atom).operator->()->*((KEYT ELEM::*) KEY)) ;
  } 
  static int compare(const void* key1, const void* key2) { 
    const KEYT* k1 = ((KEYT*)key1) ; 
    const KEYT* k2 = ((KEYT*)key2) ; 
    if ((*k1) == (*k2)) { return 0 ; } 
    else if ((*k1) > (*k2)) { return 1 ; } 
    else { return 2 ; } 
  } 
  int operator==(const DictX<ELEM, KEYT, KEY>& d) const { 
    return this->DictXGen::compare(d, extract, compare); 
  }
  int operator!=(const DictX<ELEM, KEYT, KEY>& d) const { 
    return ! (this->DictXGen::compare(d, extract, compare)) ; 
  }
  DictX<ELEM, KEYT, KEY>& operator=(const DictX<ELEM, KEYT, KEY>& d) { 
    this->clear() ; 
    this->DictXGen::insert(d,extract,compare);
    return *this ;
  }
  void insert(const DictX<ELEM, KEYT, KEY>& d) { 
    this->DictXGen::insert(d,extract,compare); 
  }
  void insert(const LocalListX<ELEM>& l) { 
    this->DictXGen::insert(l,extract,compare); 
  }
  void insert(ELEM* o) { 
    this->DictXGen::insert(o,extract,compare); 
  }
  void insertUniq(const LocalListX<ELEM>& l) {
    this->DictXGen::insertUniq(l,extract,compare); 
  }
  int  insertUniq(ELEM* o) { 
    return this->DictXGen::insertUniq(o,extract,compare); 
  }
  Ref<ELEM> getFirst(const KEYT& k) const { 
    yoo_ObjectX res = this->DictXGen::getFirst(&k, extract, compare); 
    return *(Ref<ELEM>*)&res ; 
  }
  Ref< ListX<ELEM> > getAll(const KEYT& k) const { 
    Ref< ListX<ObjectX> > res = this->DictXGen::getAll(&k, extract, compare);
    return *(Ref< ListX<ELEM> >*)&res ; 
  }
  Ref<ELEM> get(int pos) const { 
    return *(Ref<ELEM>*)this->col()->getElem(pos) ; 
  } 
  int remove(const Ref<ELEM>& o) { 
    return this->DictXGen::remove(o, extract, compare) ; 
  }
  int removeAll(const KEYT& k) { 
    return this->DictXGen::removeAll(&k, extract, compare) ; 
  }
  int member(const Ref<ELEM>& o) const { 
    return this->DictXGen::member(o, extract, compare) ; 
  }
  int key(const KEYT& k) const { 
    return this->DictXGen::indice1(&k, extract, compare) ; 
  } 
};

#endif
